#include <QEvent>
#include <QtCore>
#include <QGraphicsSceneMouseEvent>
#include "pppen.h"

PPPen::PPPen(QGraphicsItem *parent) :
    QGraphicsObject(parent)
{
    canvas = 0;
    stroke = 0;

    m_pen.setColor(Qt::red);
    m_pen.setStyle(Qt::SolidLine);
    m_pen.setWidth(3);
    m_pen.setCapStyle(Qt::RoundCap);
    m_pen.setJoinStyle(Qt::RoundJoin);
}

void PPPen::start(QGraphicsItem* canvas){
    stop();

    this->canvas = canvas;
    canvas->installSceneEventFilter(this);
}

void PPPen::paint ( QPainter * painter, const QStyleOptionGraphicsItem * option, QWidget * widget ){
    Q_UNUSED(painter);
    Q_UNUSED(option);
    Q_UNUSED(widget);
}

QRectF PPPen::boundingRect() const{
    return QRectF();
}


void PPPen::stop(){
    if (canvas)
        canvas->removeSceneEventFilter(this);
    stroke =0;
}

void PPPen::setPen(QPen &pen){
    m_pen = pen;
}

QPen PPPen::pen(){
    return m_pen;
}

QColor PPPen::color(){
    return m_pen.color();
}

void PPPen::setColor(QColor color){
    m_pen.setColor(color);
}

void PPPen::setWidth(int val){
    m_pen.setWidth(val);
}

int PPPen::width(){
    return m_pen.width();
}


bool PPPen::sceneEventFilter ( QGraphicsItem * watched, QEvent * event){
    bool handled = false;
    QGraphicsSceneMouseEvent *mouseEvent;
    if (watched == canvas) {
        if (event->type() == QEvent::GraphicsSceneMousePress ||
            event->type() == QEvent::GraphicsSceneMouseDoubleClick
            ) {
            mouseEvent = static_cast< QGraphicsSceneMouseEvent *>(event);
            stroke = new PPGraphicsStroke(watched);
            stroke->setPen(m_pen);
            stroke->append(mouseEvent->pos());
//            stroke->append(mouseEvent->pos());
            handled = true;

        } else if (event->type() == QEvent::GraphicsSceneMouseRelease ) {
            mouseEvent = static_cast< QGraphicsSceneMouseEvent *>(event);
//            stroke->append(mouseEvent->lastPos());
            stroke->append(mouseEvent->pos());
            handled = true;

            canvas->removeSceneEventFilter(this);
            stroke = 0;
            emit added();

        } else if (event->type() == QEvent::GraphicsSceneMouseMove ) {
            mouseEvent = static_cast< QGraphicsSceneMouseEvent *>(event);

//            stroke->append(mouseEvent->lastPos());
            stroke->append(mouseEvent->pos());
            handled = true;

        }
    }

    if (!handled)
        return QGraphicsObject::sceneEventFilter(watched,event);
    return handled;
}
